/* FL.c            ---------------------------------------------- */
/* File Lister     - gestione interattiva ambiente Unix           */
/*                                                                */
/* - Visualizza la directory full screen                          */
/* - Permette la selezione di file / directory                    */
/* - Permette di dare comandi relativi al file selezionato        */
/*                                                                */
/*                                                                */
/* Es. compilazione:       cc fl.c -ofl -lcurses -DAIX            */
/*                                                                */
/*                                                     Piero 9/98 */
/* -------------------------------------------------------------- */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <unistd.h>
#include <sys/types.h>

#ifdef AIX
   #include <curses.h>                  /* ambiente Unix standard  */
#else  
   #include <ncurses/curses.h>          /* ambiente Linux */
#endif

#define  MAXLINE  200           /* lunghezza massima riga di file directory */
#define  MAXSTR   80            /* lunghezza massima stringa */

/* Variabili globali ----------------------------- */

char filetemp[MAXSTR], loctemp[MAXSTR], arctemp[MAXSTR], ftptemp[MAXSTR]; 
char msgtemp[MAXSTR],  prmtemp[MAXSTR]; 
char record[MAXLINE];
FILE *ifp;             /* file directory                                  */
FILE *pfp;             /* file parametri per ftp                          */
FILE *mfp;             /* file messaggi  per ftp                          */
int nrec, nrecold;     /* contatore dei record letti                      */
int numrighe = 25;     /* numero di righe dello schermo                   */
int numcol   = 80;     /* numero di colonne dello schermo                 */ 
int r1, c1, r2, c2;    /* righe e colonne                                 */
chtype chc0[10];       /* caratteri della cornice attuale                 */
  /* tipo di cornice n. 1 */
chtype chc1[10] = {'|', '-', '+', '+', '+', '+', '|', '|', '-', '-'};  
  /* tipo di cornice n. 2       */
chtype chc2[10] = {0x000000b3,0x000000c4,0x000000da,0x000000bf,0x000000c0,
       0x000000d9,0x000000c3,0x000000b4,0x000000c2,0x000000c1};

char fullscr = 0;         /* indica se e' a 2 colonne (fullscr=0) o a 1   */
char typec = 1;           /* indicatore tipo di cornice                   */
char fillbox = 0;         /* = 0 per comando BOX, = 1 per WINDOW          */ 
chtype attr, mask;        /* per gestione attributi                       */
int  gsett, gmese, mese;  /* giorno della settimana, del mese e num. mese */
char ora[6], anno[6];     /* ore:minuti, anno                             */
char dateday[9];          /* data nella forma gg/mm/aa                    */
char nomevar[25];         /* nome eventuali variabili di shell            */
char totale[20];          /* stringa con totale occupazione directory     */
char workdir[MAXSTR];     /* directory attuale                            */
char homedir[MAXSTR];     /* home directory                               */
char maskfile[60];        /* maschera di selezione files                  */
int  long ptrlined[2000]; /* fino a 2000 entrate nel file directory       */
char filetag[2000];       /* indicatore di file selezionato con barra spaz*/
int  nlinepag = 20;       /* num. massimo linee di elenco per videata     */
int  linecurs = 1;        /* linea occupata dal cursore (in reverse)      */
int  linedirprec = 1;     /* linea occupata dal curs. nella dir. precedente*/
int  nrecdirprec = 1;     /* linea iniziale videata nella dir. precedente */
int  tagged   = 0;        /* Numero files selezionati con barra spazio    */
/* char errline[MAXSTR]; */ 
int  tipovis;       /* 0 = visualizza grandezza, 1 data, 2 auth, 3 = own. */
int  totfiledir;          /* numero totale di files nella directory       */
char nomeuser[16];        /* nome utente                                  */
char minsec[6];           /* stringa contenente minuti e secondi          */
char sortflag[4];         /* flags di sort per comando ls                 */
int  arcflag;             /* flag di display .tar o .zip                  */
int  sessionflag;         /* flag indicatore di sessione attiva: 1= FTP   */
int  ftpflag;             /* flag di FTP attivo, 1= si                    */
char ftpworkdir[MAXSTR];  /* directory remota corrente                    */  
char ftpcomando[MAXSTR];  /* comando ftp (get, put, ecc.)                 */  
char ftpcomando1[MAXSTR]; /* comando ftp (get, put, ecc.)                 */  
char ftphost[40];         /* nome host remoto                             */
char ftpuser[16];         /* nome utente per logon con FTP                */
char ftppassw[16];        /* password per logon con FTP                   */         

/* Campi raccolti con 'ls -al' che occupano una videata */
char filename[42][61];      /* 40 nomi file lunghi al massimo 60          */
char filesize[42][11];      /* grandezza files in bytes                   */
char filemonth[42][5];      /* mese ultimo aggiornamento files            */
char fileday[42][5];        /* giorno ultimo aggiornamento files          */
char fileyear[42][6];       /* anno/ora ultimo aggiornamento files        */
char fileauth[42][11];      /* autorizzazione                             */       
char fileown[42][10];       /* owner e group                              */
char filegroup[42][10];     /* group                                      */
char filelink[42][4];       /* eventuale freccia che indica un link       */
char fileresto[42][61];     /* eventuale specifica al file puntato dal link*/

/* Stringhe impostabili con il Setup                      */
char CLEAR[16] = "clear ";
#ifdef AIX
   char PAGER1[16] = "most ";
   char PAGER2[16] = "pg ";
#else  
   char PAGER1[16] = "less ";  
   char PAGER2[16] = "more ";
#endif
char EDITOR1[16] = "vi ";
char EDITOR2[16] = "joe ";
char UNTAR[16]  = "tar -tv";    /* <-- verra' aggiunto il flag 'f'   */
char UNZIP[16]  = "unzip -lv ";
char UNCOMP[16] = "zcat ";
char UNGZIP[16] = "gzip -dc ";

void hlp_scr(); /* prototipo della funzione help screen che e' in fondo */
 
/* Display linea in riga 23 */
void display23 (char *str)
{
    int i;
    
    for (i = 0; i < 80 ; i++) mvaddch(23, i, ' '); 
    attrset (A_REVERSE);
    mvaddstr (23, 0, str); 
    refresh();
    attrset (A_NORMAL);
}
 

/* Scrive una cornice in stdscr; input: r1,c1,r2,c2, typec, fillbox  */
void cornice(int r1, int c1, int r2, int c2)
{
    int i,j;
     
    if (typec == 1)  for (i=0;i<=9;i++) chc0[i] = chc1[i];    
    if (typec == 2) { 
       for (i=0;i<=9;i++) chc0[i] = chc2[i];  
       attr = A_ALTCHARSET ; 
       attrset (attr); 
    }  
       
    /* per ogni riga della finestra da delimitare     */
    for (i = r1 ; i <= r2 ; i++ ) {
       if (i == r1) mvaddch(r1, c1, chc0[2] );
       else if (i == r2) mvaddch(r2, c1, chc0[4]);
            else mvaddch(i, c1, chc0[0] );
       
       /* fa il bordo o il centro     */
       for (j = c1+1 ; j <= c2-1 ; j++ ) {
           if ((i == r1) || (i == r2)) addch(chc0[1] );   
           else  if ( fillbox == 1)  addch(' ');
       }

       /* completa il bordo destro */
       if (i == r1) mvaddch(r1, c2, chc0[3] );
       else if (i == r2) mvaddch(r2, c2, chc0[5] );
            else mvaddch(i, c2, chc0[0] );
            
    }
    
    attrset (A_NORMAL ); 
    
}    

/* Estrae data e ora        */
void data()
{
    time_t t;
    char wd[22] = "SunMonTueWedThuFriSat"; 
    char mo[38] = "JanFebMarAprMayJunJulAugSepOctNovDec";
    char s[30], ss[4] ; int i;
    
    time(&t);             /* estrae timestamp    */ 
    strcpy (s,ctime(&t)); /* trasforma timestamp in stringa: Set Mes nn hh:mm:ss yyyy */ 

    strncpy (ss, s, 3); ss[3] = 0;
    gsett = ( strstr(wd, ss) - wd ) / 3; gsett++;  /* 1 = Domenica   */
    strncpy (ss, s+4, 3);
    mese = ( strstr(mo, ss) - mo ) / 3;  mese++;
    strncpy (ss, s+8, 3);
    gmese =  atoi(ss); strncpy(ora, s+11, 5); strncpy(anno, s+20, 4);
    if ((gsett<1) || (gsett>7)) gsett = 0;
    if ((mese<1) || (mese>12)) mese = 0;
    sprintf(dateday, "%2d/%02d/%c%c", gmese, mese, anno[2], anno[3]);
    strncpy(minsec, s+14, 2); strncpy(minsec+2, s+17, 2); minsec[4] = 0; 
  
}

/* Produce la directory in un file temporaneo e raccoglie:        */
/*         - i puntatori ai record: ptrlined[]                    */
/*         - l'elenco di sottodirectory in elencodir              */

void getdir()
{
    char c, stringa[MAXLINE], comando[MAXSTR];
    int  n = 0;

    if ( sessionflag == 1) strcpy (filetemp, ftptemp);
    else {
       strcpy (filetemp, loctemp);
       if ( getcwd(workdir, MAXSTR) == NULL) {   
          fprintf(stderr,"FL - Errore in 'getcwd'.\n");
          exit (98);
       }
       strcpy(comando, "ls -alF");
       strcat(comando, sortflag);
       /*  strcat(comando, maskfile); */
       strcat(comando, " > ");
       strcat(comando, filetemp);
       if ( system(comando) != 0) {
          fprintf(stderr,"FL - Errore in comando: %s.\n", comando);
          exit (98);
       }
    }
    
    if ( (ifp = fopen(filetemp, "r")) == NULL) {
       fprintf(stderr,"FL - Errore in apertura file %s.\n",filetemp);
       exit (97);
    }

    /* legge il file directory fino alla fine */
    while ( fgets(record, MAXLINE, ifp) != NULL ) {
       if ( n == 0) strcpy(totale, record);
       ptrlined[n] = ftell(ifp);
       n ++;
        
     /*  printf(" %d %s", ptrlined[n], record); */
    }
    fclose(ifp); 
    
    totfiledir = n;
    /* linecurs = 1;  * posizione linea in reverse    */
    /* nrec = 1; * puntatore a record in file temporaneo, corr. a prima riga video */
    nrecold = 0;
}

/* Legge 40 record dal file temporaneo a partire da riga 'nrec' */
/* riempie i vettori 'filename' 'filesize' ecc.                         */

void videofill()
{
    int i, j, l, nfields, flag, devflag;
    char stringa[16], *cp;
   
    if (totfiledir < nrec) { nrec = totfiledir; linecurs = 1; }
    if (nrec+linecurs >= totfiledir) linecurs = 1;
    if (nrec == nrecold) return;   
    nrecold = nrec; j = 0;  

    if (fullscr == 0) nlinepag = 40; else nlinepag = 20;
   
    if ( (ifp = fopen(filetemp, "r")) == NULL) {
       fprintf(stderr,"FL - Errore in apertura file %s.\n",filetemp);
       exit (96);
    }
    flag = 0; i = 0;
    fseek (ifp, ptrlined[nrec], SEEK_SET);
    /* legge il file directory dal record 'nrec' per 'nlinepag' */
    while ( fgets(record, MAXLINE, ifp) != NULL ) {
      /* tiene conto se e' un device: se si, ha un campo in piu' */
       if ( (record[0] == 'c') || (record[0] == 'b') ) {
          if ((cp = strstr(record,",")) != NULL) *cp = ' ';
          devflag = 1;
       }
       else devflag = 0;
       strcpy (filename[j], "");
       strcpy (filelink[j], "");
       strcpy (fileresto[j], "");
      /* tiene conto che in 'ls -l' puo' mancare il gruppo */
       if (flag == 0) {
          if (devflag == 0 )
          nfields = sscanf (record, "%s %*s %s %s %s %s %s %s %s %s %s",
            &fileauth[j], &fileown[j], &filegroup[j], &filesize[j],
            &filemonth[j], &fileday[j], &fileyear[j], &filename[j],
            &filelink[j], &fileresto[j]); 
          else
          nfields = sscanf (record, "%s %*s %s %s %*s %s %s %s %s %s %s %s",
            &fileauth[j], &fileown[j], &filegroup[j], &filesize[j],
            &filemonth[j], &fileday[j], &fileyear[j], &filename[j],
            &filelink[j], &fileresto[j]); 
       }
       if ( (nfields == 7) && (i == 0) ) flag = 1;
       if (flag == 1) { 
          if (devflag == 0 )
          nfields = sscanf (record, "%s %*s %s %s %s %s %s %s %s %s",
            &fileauth[j], &fileown[j], &filesize[j],
            &filemonth[j], &fileday[j], &fileyear[j], &filename[j],
            &filelink[j], &fileresto[j]); 
          else
          nfields = sscanf (record, "%s %*s %s %s %*s %s %s %s %s %s %s %s",
            &fileauth[j], &fileown[j], &filesize[j],
            &filemonth[j], &fileday[j], &fileyear[j], &filename[j],
            &filelink[j], &fileresto[j]); 
          strcpy (filegroup[j], "");
       }
       i++;
       j++;
       if (j >= nlinepag) break;    
    }
    fclose(ifp); 
    for (i = j ; i < nlinepag ; i++) {
       strcpy( filename[i], "                                ");
       strcpy( filesize[i], "          "); 
       strcpy( filemonth[i],"    "); 
       strcpy( fileday[i],  "    "); 
       strcpy( fileyear[i], "     "); 
       strcpy( fileauth[i], "          "); 
       strcpy( fileown[i],  "         "); 
       strcpy( filegroup[i],"         "); 
    }
}


/* Riempie la schermata da vettori file* e tipovis                */

void videoout()
{
   char filetrunc[40], stringa[40], lineablank[MAXSTR];
   int  i,l, col = 0, riga = 0;
   

    strcpy (lineablank, "                                                                  ");
    /* Scrive 2 linee orizzontali */          
    attrset(A_NORMAL); 
    /* attrset(A_ALTCHARSET); */ 
    for (i = col ; i < col+79; i++) {
       mvaddch(0, i, chc0[1]);  
       mvaddch(2, i, chc0[1]); 
    }

    /* Scrive cornici e intestazioni     */
    mvaddstr(0, 1, "User: ");
    if (sessionflag == 0) mvaddstr(0, 7, nomeuser);
    else { strcpy(stringa,ftpuser); strcat(stringa,"@"); 
       strcat(stringa, ftphost); mvaddstr(0, 7, stringa); }
    mvaddstr(0, 65, dateday); 
    mvaddstr(0, 74, ora); 
    attrset(A_NORMAL | A_REVERSE);
    mvaddstr(0, 32, " File Lister "); 

   attrset (A_NORMAL | A_BOLD );
   mvaddstr ( 1, col+1, lineablank ); 
   if (sessionflag == 0) mvaddstr ( 1, col+1, workdir );
   else mvaddstr ( 1, col+1, ftpworkdir );
   attrset (A_NORMAL);
   sprintf(stringa, "%4d files", totfiledir-3);
   mvaddstr ( 1, col+69, stringa);
   attrset (A_NORMAL | A_BOLD);
   mvaddstr ( 2, col+1, "Files ");
   mvaddstr ( 2, col+41, "Files ");

   if (fullscr == 0) {
      if (tipovis == 0) { mvaddstr(2, col+34, " Size");
                          mvaddstr(2, col+74, " Size"); } 
      if (tipovis == 1) { mvaddstr(2, col+26, " Last Update");
                          mvaddstr(2, col+66, " Last Update"); } 
      if (tipovis == 2) { mvaddstr(2, col+28, " Authorize "); 
                          mvaddstr(2, col+68, " Authorize "); }
      if (tipovis == 3) { mvaddstr(2, col+21, " Owner    Group  "); 
                          mvaddstr(2, col+61, " Owner    Group  "); } 
   }
   else { 
      mvaddstr(2, col+34, " Size"); 
      mvaddstr(2, col+40, "Last Update"); 
      mvaddstr(2, col+53, " Authorize"); 
      mvaddstr(2, col+63, " Owner    Group ");
   }    


   if (fullscr == 0) nlinepag = 40; else nlinepag = 20;
 
   for (i = 0 ; i < nlinepag ; i++) {
       if (i >= 20) { col = 40; riga = i - 20; }
       else riga = i;
       if (i == linecurs-1) attrset (A_REVERSE);
       else  attrset (A_NORMAL ); 
       mvaddstr(riga+3, col, "                             "); 
       l = strlen(filename[i]);
       strncpy(filetrunc, filename[i], 36);
       if ((strlen(filelink[i]) > 0) && (l < 34) ) {
          strcat (filetrunc, " ");
          strcat (filetrunc, filelink[i]);
          strcat (filetrunc, " ");
          if (l < 34) strncat (filetrunc, fileresto[i], 34-l);
       }
       l = 10 - strlen(filesize[i]);
       if (fileauth[i][0] == 'd')  mvaddch(riga+3, col, '+' | A_BOLD);
       /* if (filetag[nrec+i] == '>')  mvaddch(riga+3, col, '>' | A_BOLD); */
       if (fullscr == 0) {
          mvaddstr(riga+3, col+22, "                  "); /* 18 blank */
          mvaddstr(riga+3, col+1, filetrunc);
          if (tipovis == 0) {
             mvaddstr(riga+3, col+29+l, filesize[i]);
          }
          if (tipovis == 1) {
             mvaddstr(riga+3, col+27, fileday[i]); 
             mvaddstr(riga+3, col+30, filemonth[i]); 
             mvaddstr(riga+3, col+34, fileyear[i]);
          }    
          if (tipovis == 2) mvaddstr(riga+3, col+29, fileauth[i]); 
          if (tipovis == 3) {
             mvaddstr(riga+3, col+22, fileown[i]); 
             mvaddstr(riga+3, col+31, filegroup[i]);
          }    
       }
       else {
          mvaddstr(riga+3, col+29, lineablank);
          mvaddstr(riga+3, col+1,  filetrunc);
          mvaddstr(riga+3, col+29+l, filesize[i]);
          mvaddstr(riga+3, col+40, fileday[i]); 
          mvaddstr(riga+3, col+43, filemonth[i]); 
          mvaddstr(riga+3, col+47, fileyear[i]); 
          mvaddstr(riga+3, col+53, fileauth[i]); 
          mvaddstr(riga+3, col+64, fileown[i]);
          mvaddstr(riga+3, col+73, filegroup[i]);
       }          
   }
   attrset (A_REVERSE);
   mvaddstr ( 23, 0, " Help = ?/F1    Commands = a g c d r m l L e E o p i s t !       Exit = q / F10");
   if (sessionflag != 0)  
   mvaddstr ( 23, 0, " W  switch FTP session -     Commands = w a d r l L p s n y       Exit = q/F10 " );

   if (fullscr == 0) {
      /* Scrive la linea centrale               */
      /* attrset(A_ALTCHARSET);  */
      for (i = 3 ; i <= 22 ; i++ )  mvaddch(i, 39 , chc0[0]);
      attrset(A_NORMAL); 
   } 
 /*  else {
      attrset(A_NORMAL); 
      for (i = 3 ; i <= 22 ; i++ )  mvaddch(i, 39 , ' ');
   } */  
 }


/* Fa partire il transfer file FTP in maniera batch                */

int runftp()
{
   int  rc; char c, *cp1, *cp2;
   char comando[MAXSTR], msgrec[MAXSTR];
   
       rc = 0;
       if ( (pfp = fopen(prmtemp, "w")) == NULL) {
          display23 (" Errore in creazione file parametri FTP. (premi ENTER)");
          c = getch();
          return 1;
       }
       fprintf (pfp, "user %s %s\n", ftpuser, ftppassw);
       fprintf (pfp, "cd %s\n",ftpworkdir);
       fprintf (pfp, "pwd\n");
       if (strlen(ftpcomando1) > 0) fprintf(pfp, "%s\n", ftpcomando1);
       fprintf (pfp, "%s\n", ftpcomando);
       fprintf (pfp, "bye\n");
       fclose (pfp);
       strcpy (ftpcomando1, "");

       move (23, 0); refresh();
       strcpy (comando, "ftp -in ");
       strcat (comando, ftphost);
       strcat (comando, " < ");
       strcat (comando, prmtemp);
       strcat (comando, " > ");
       strcat (comando, msgtemp);
       move (23, 0); refresh();
       if ((rc = system (comando)) != 0) {
           display23(" Errore in lancio FTP.                    (premi ENTER) ");
           c = getch();
           goto end_runftp;
       }   

       /* legge la home directory remota dal file messaggi  */
       if ( (mfp = fopen(msgtemp, "r")) == NULL) {
          display23 (" Errore in lettura workdir remota         (premi ENTER)");
          c = getch();
          rc = 2;
          goto end_runftp;
       }
       /* e la mette nella variabile  'ftpworkdir'   */
       while ( fgets(msgrec, MAXLINE, mfp) != NULL ) {
          if ((cp1 = strstr(msgrec, "\"/")) != NULL) {
             cp2 = strstr(cp1+2, "\"");
             if (cp2 > cp1) {
                strcpy(ftpworkdir, ++cp1);
                ftpworkdir[cp2-cp1] = 0;
                break;
             }
          }
       }
       fclose (mfp);
           
end_runftp:  ;

       if (rc != 0) {
          strcpy (comando, "rm ");
          strcat (comando, prmtemp);
          system (comando);
          clear();
       }
       return rc;
}

/* In sessione ftp, a seconda dell'input, fa CD remoto o lista il file */

void ftp_list(char *stringa, char auth)
{   
     int  l, lw, i;
     char c, comando[MAXSTR], s[40];
     char *cp;

     l = strlen(stringa) - 1;
     lw = strlen(ftpworkdir);

     move (23, 0); refresh();
     /* se il file non e' directory, fa transfer e lo visualizza */     
     if (auth == '-')  {    /* e' file normale */

        if ( (strstr(stringa, ".tar") != NULL) ||
             (strstr(stringa, ".zip") != NULL) ||
             (strstr(stringa, ".Z") != NULL) ||
             (strstr(stringa, ".gz") != NULL) ) {
             display23 (" Non puoi vedere un archivio o un file compresso in remoto!  (premi ENTER)");
             c = getch();
             return;
        }     
        strcpy (ftpcomando, "get  ");
        strcat (ftpcomando, stringa);
        strcat (ftpcomando, " ");
        strcat (ftpcomando, arctemp);
 
        if (runftp() == 0) {
           strcpy (comando, PAGER1);
           strcat (comando, arctemp);
           system (CLEAR); 
           system (comando);
           clear();
        }   
        else {
           display23 (" Errore in sessione FTP !!  (premi ENTER)");
           c = getch();
           strcpy (comando, "rm ");
           strcat (comando, prmtemp);
           system (comando);
           clear();
           return;
        }

        strcpy (comando, "rm ");
        strcat (comando, arctemp);
        system (comando);
        return;
     }

     /* Se e' una directory, da il comando CD      */ 
     if ((strncmp (stringa, "..", 2) == 0) && (lw > 1))  {  
        for (i = lw-2; i >= 0; i--) {
            if (ftpworkdir[i] == '/') {
               ftpworkdir[i] = 0;
               if (strlen(ftpworkdir) == 0) strcpy(ftpworkdir, "/");
               break;
            }
        }
        strcpy (ftpcomando, "nlist -alF ");
        strcat (ftpcomando, ftptemp);
     }
     else {
        if ((auth == 'd') || (stringa[l] == '/')) {    /* e' directory */
           if ( ftpworkdir[lw-1] != '/' ) strcat (ftpworkdir, "/");
           strcat (ftpworkdir, stringa);

           strcpy (ftpcomando, "nlist -alF ");
           strcat (ftpcomando, ftptemp);
        }
     }
     if (runftp() == 0) {
        ftpflag = 1;
        sessionflag = 1;
        getdir();  linecurs = 1; nrec = 1;
        videofill();
        clear();
     }   
     else {
        display23 (" Errore in sessione FTP !!  (premi ENTER)");
        c = getch();
        strcpy (comando, "rm ");
        strcat (comando, prmtemp);
        system (comando);
        clear();
     }

}
 
/* Input da tastiera di una stringa lunga 'l', con possibilita' di editing */
/* Si presuppone l'uso di curses, e che sia preventivamente dato initscr, cbreak */

void inputtext(char *str, int l, char tipo)

{   
    /*  'tipo' puo' essere: '9' solo cifre numeriche 0-9;
                            'A' solo alfanumerico (lettere+numeri)
         campi al massimo lunghi 80                          */
    
    int i, j, n; 
    int c; 
    char stringa[80];
    
    noecho(); 
    keypad (stdscr, TRUE);    
    tipo = toupper(tipo);
    n = strlen(str);
    if ((l < 0) || (l > 80)) l = 80;    
    for (i = 0 ; i < l ; i++) {
       if (i < n ) c = str[i];
       else c = ' '; 
       addch (c);
       stringa[i] = c;
    }   
    
    for (i = 0 ; i < l ; i++) addch('\b'); 

    i = 0;
  itera:   
    refresh(); 
    c = getch();  
    if (c == '\n') {  /* se e' premuto ENTER, esce */
       stringa[n] = 0; 
       strcpy(str, stringa);
       echo();
       return; 
    }     
    if (c == KEY_RIGHT) { 
       if (i < l) { 
          i++;
          addch(stringa[i-1]);
          goto itera;
       }   
       else goto errore;
    }    
    if (c == KEY_DC)  {   /* delete character */
       if (n > i) {
          n--;
          strcpy(stringa+i,stringa+i+1); stringa[l-1] = ' ';
          for (j = i ; j < l ; j++) addch(stringa[j]); 
          for (j = i ; j < l ; j++) addch('\b'); 
          goto itera;
       } 
       else goto errore;
    }     
    if ((c == KEY_BACKSPACE) || (c == KEY_LEFT)) { 
       if (i > 0) {
          i--; 
          addch('\b');
          goto itera;
       } else goto errore;  
    }   
    if ((c == KEY_DOWN) || (c == KEY_UP) || (c == KEY_HOME) || (iscntrl(c)) ||
       (c == KEY_NPAGE) || (c == KEY_PPAGE) || (c == KEY_DC)) goto errore;
    if ((tipo == '9') && ! isdigit(c)) goto errore;   
    if ((tipo == 'A') && ! (isdigit(c) || isalpha(c))) goto errore;   
    if (i >= l) goto errore;
    stringa[i] = c; 
    i++; 
    if (i > n) n = i;
    addch(c);
    goto itera;
  errore:
    beep(); 
    goto itera;
}


int main(int argc, char *argv[])
 
{  
    int i, j, l, n; 
    int c, uc;
    char stringa[MAXSTR], stringa1[MAXSTR], comando[MAXSTR];
    char *cp;
   
    strcpy(nomevar, ""); strcpy(record, ""); 
    mask = A_NORMAL; 
          
    data();           /* estrae data, ora giorno del mese e della settimana */

    #ifdef AIX
    /* Caratteri cornice singola per AIX      */
    chc2[0] = 'x';    chc2[1] = 'o';     chc2[2] = 'l'; 
    chc2[3] = 'k';    chc2[4] = 'm';     chc2[5] = 'j';
    chc2[6] = 't';    chc2[7] = 'u';     chc2[8] = 'w';    chc2[9] = 'v'; 
    #else  /* Caratteri cornice per ncurses Linux   */
    chc2[0] = ACS_VLINE;    chc2[1] = ACS_HLINE;   chc2[2] = ACS_ULCORNER; 
    chc2[3] = ACS_URCORNER; chc2[4] = ACS_LLCORNER;  chc2[5] = ACS_LRCORNER;
    #endif

    for (i = 0; i < 10 ; i++) chc0[i] = chc1[i];
    for (i = 0; i < 2000 ; i++) filetag[i] = '\0';
    
    /* inizializza curses e variabili    */
    initscr();
    cbreak();
    noecho(); 
    keypad (stdscr, TRUE);
    clear();
        
    tipovis  = 0;  /* visualizza grandezza files    */ 
    nlinepag = 20; /* numero linee per pagina       */
    strcpy (nomeuser, getenv("LOGNAME"));
    strcpy (homedir,  getenv("HOME"));
    strcpy (sortflag, ""); arcflag = 0;
    strcpy (ftpworkdir, "."); ftpflag = 0; sessionflag = 0;
    strcpy (ftphost, ""); strcpy (ftpuser, nomeuser);
    strcpy (ftppassw, ""); 
    strcpy (ftpcomando, "");
    strcpy (ftpcomando1, "");
    
    /* Nome file temporaneo contenente l'output di "ls "   */
    strcpy (filetemp, homedir);
    strcat (filetemp, "/tmp~");
    strcat (filetemp, minsec);
    strcpy (arctemp, filetemp);
    strcpy (ftptemp, filetemp);
    strcpy (prmtemp, filetemp);
    strcpy (msgtemp, filetemp);
    strcat (filetemp, ".dir");
    strcpy (loctemp, filetemp);
    /* Nome file temporaneo contenente l'output di "tar -t " o "unzip -l"   */
    strcat (arctemp, ".arc");
    /* Nome file temporaneo contenente i parametri dell'FTP   */
    strcat (prmtemp, ".prm");
    /* Nome file temporaneo contenente l'output di "nlist -l" con l'FTP   */
    strcat (ftptemp, ".ftp");
    /* Nome file temporaneo contenente l'output dei messaggi dell'FTP   */
    strcat (msgtemp, ".msg");

    strcpy (maskfile, "*");

    /* ottiene files directory corrente  */
    getdir(); linecurs = 1; nrec  = 1;
    

    nrec = 1; /* puntatore a record in file temporaneo, corr. a prima riga video */
    nrecold = 0;
    /* Scrive elenco files               */
    videofill();
    
    /* Main loop di rilevamento tasti premuti e azioni conseguenti  */
    for (;;) {
       noecho(); 
       videoout();
       refresh();     
       c = getch();  
       if ( (toupper(c) == 'A') && (fullscr == 0) ) { 
          if (tipovis < 3) tipovis++;
          else tipovis = 0;
        /*  clear(); */
          goto loop;
       }    
       if ( (c == KEY_RIGHT) && (fullscr == 0) ) { 
          if (linecurs + 20 < totfiledir - nrec) { 
             linecurs += 20;
          }   
       /*   if (tipovis < 3) tipovis++;
          else tipovis = 0;
          clear(); */
       }    
       if ( (c == KEY_LEFT) && (fullscr == 0)) { 
          if (linecurs > 20) { 
             linecurs -= 20;
          }   
       /*   if (tipovis > 0) tipovis--;
          else tipovis = 3;
          clear(); */
       }   
       if ((c == KEY_NPAGE) || (toupper(c) == 'F')) { 
          if (nrec+nlinepag-1 < totfiledir) { 
             nrec = nrec+19;
             if (nrec+linecurs > totfiledir) linecurs = 1;
             videofill();
          }   
          else linecurs = totfiledir - nrec - 1;
          goto loop;
       }    
       if ((c == KEY_PPAGE) || (toupper(c) == 'B')) { 
          if (nrec > 1) { 
             nrec = nrec-19;
             if (nrec < 1 ) nrec = 1;
             videofill();
          }     
          else linecurs = 1;
          goto loop;
       }   
       if ((c == KEY_DOWN) || (c == KEY_SR) || (toupper(c) == 'J')) { 
          if ((linecurs >= nlinepag) && (nrec+nlinepag+1 < totfiledir)) nrec++;
          else 
             if ((linecurs < nlinepag) && (nrec+linecurs+1 < totfiledir))
                 linecurs++;
          videofill();
          goto loop;
       }    
       if ((c == KEY_UP) || (c == KEY_SF) || (toupper(c) == 'K')) { 
          if (linecurs > 1 ) linecurs--;
          else if (nrec > 1) nrec--;
          videofill();  
          goto loop;
       }   
       if ( (c == KEY_HOME) || (c == KEY_A1) ) {
          nrec = 1; 
          videofill();
          goto loop;
       }   
       if (toupper(c) == 'L') {    /* se e' premuto 'l' o 'L'*/
          n = linecurs-1;
          sscanf(filename[n], "%s", stringa);
          l = strlen(filename[n])-1;
          uc = filename[n][l];
          l = strlen(stringa)-1;
          if ((fileauth[n][0] == 'c') || (fileauth[n][0] == 'b')) {
             display23(" Non puoi visualizzare un driver !! (premi ENTER) ");
             c = getch();
             goto loop;
          }
          if ((stringa[l] == '*') || (stringa[l] == '/')) stringa[l] = 0; 
          if ((ftpflag == 1) && (sessionflag == 1)) {
             if (atoi(filesize[n]) > 1000000) {
                 display23(" File troppo grande per visualizzarlo in remoto !!   (premi ENTER) ");
                 c = getch();
                 goto loop;
             } 
             ftp_list(stringa, fileauth[n][0]);
             goto loop;
          }
          if ((fileauth[n][0] == 'd') || (uc == '/')) {   /* e' directory */
             if (chdir(stringa) == 0) {             
                if (strncmp(stringa, "..", 2) != 0 ) {
                   linedirprec = linecurs;
                   nrecdirprec = nrec;
                   getdir (); linecurs = 1; nrec = 1;
                }
                else {
                   getdir ();
                   linecurs = linedirprec;
                   nrec = nrecdirprec;
                   linedirprec = 1;
                }
                tagged = 0; for (i = 0; i < 2000 ; i++) filetag[i] = '\0';
                videofill();
             }
          }
          else  {
             if ( c == 'L' ) strcpy (comando, PAGER2);
             else strcpy (comando, PAGER1);
             strcat (comando, stringa);
             system (CLEAR);
             system (comando);
          }
          clear();
          goto loop;
       }
       if (c == '\n')  {    /* se e' premuto ENTER */
          n = linecurs-1;
          sscanf(filename[n], "%s", stringa);
          l = strlen(filename[n])-1;
          uc = filename[n][l];
          l = strlen(stringa)-1;
          if ((stringa[l] == '*') || (stringa[l] == '/')) stringa[l] = 0; 
          if ((fileauth[n][0] == 'c') || (fileauth[n][0] == 'b')) {
             display23(" Non puoi visualizzare un driver !! (premi ENTER) ");
             c = getch();
             goto loop;
          }
          if ((ftpflag == 1) && (sessionflag == 1)) {
             if (atoi(filesize[n]) > 1000000) {
                 display23(" File troppo grande per visualizzarlo in remoto !!   (premi ENTER) ");
                 c = getch();
                 goto loop;
             } 
             ftp_list(stringa, fileauth[n][0]);
             goto loop;
          }
          if ((fileauth[n][0] == 'd') || (uc == '/')) {    /* e' directory */
             if (chdir(stringa) == 0) {             
                if (strncmp(stringa, "..", 2) != 0 ) {
                   linedirprec = linecurs;
                   nrecdirprec = nrec;
                   getdir (); linecurs = 1; nrec = 1;
                }
                else {
                   getdir ();
                   linecurs = linedirprec;
                   nrec = nrecdirprec;
                   linedirprec = 1;
                   nrecdirprec = 1;
                }
                tagged = 0; for (i = 0; i < 2000 ; i++) filetag[i] = '\0';
                videofill();
             }
          }
          else  {
             move (23, 0); refresh();
             if ((cp = strstr(filename[n], ".tar")) != NULL) {
                strcpy (comando, UNTAR);
                if ( (cp = strstr(filename[n], ".tar.Z")) != NULL) strcat (comando, "zf ");
                else if ( (cp = strstr(filename[n], ".tar.gz")) != NULL) { 
                        strcat (comando, "Zf ");
                     } 
                     else strcat (comando, "f ");   
                if ( (cp - filename[n]) >= (l - 7)) { 
                   strcat (comando, filename[n]);
                   strcat (comando, " > ");
                   strcat (comando, arctemp);
                   if (system (comando) != 0) {
                       display23(" Errore in TAR -t       (premi ENTER) ");
                       c = getch();
                       goto loop;
                   } 
                   strcpy (stringa, arctemp);
                   arcflag = 1;  
                }
                goto vedi;
             }   
             if ((cp = strstr(filename[n], ".zip")) != NULL) {
                if ( (cp - filename[n]) >= (l - 4)) { 
                   strcpy (comando, UNZIP);
                   strcat (comando, filename[n]);
                   strcat (comando, " > ");
                   strcat (comando, arctemp);
                   if (system (comando) != 0) {
                       display23(" Errore in UNZIP -l       (premi ENTER) ");
                       c = getch();
                       goto loop;
                   } 
                   strcpy (stringa, arctemp);
                   arcflag = 1;  
                }
             }
             if ((cp = strstr(filename[n], ".Z")) != NULL) {
                if ( (cp - filename[n]) >= (l - 2)) { 
                   strcpy (comando, UNCOMP);
                   strcat (comando, filename[n]);
                   strcat (comando, " > ");
                   strcat (comando, arctemp);
                   if (system (comando) != 0) {
                       display23(" Errore in ZCAT         (premi ENTER) ");
                       c = getch();
                       goto loop;
                   } 
                   strcpy (stringa, arctemp);
                   arcflag = 1;  
                }
             }
             if ((cp = strstr(filename[n], ".gz")) != NULL) {
                if ( (cp - filename[n]) >= (l - 3)) { 
                   strcpy (comando, UNGZIP);
                   strcat (comando, filename[n]);
                   strcat (comando, " > ");
                   strcat (comando, arctemp);
                   if (system (comando) != 0) {
                       display23(" Errore in GZIP         (premi ENTER) ");
                       c = getch();
                       goto loop;
                   } 
                   strcpy (stringa, arctemp);
                   arcflag = 1;  
                }
             }
             vedi:  ;
             strcpy (comando, PAGER1);
             strcat (comando, stringa);
             system (CLEAR); 
             system (comando);
             if (arcflag == 1) { 
                arcflag = 0;
                strcpy (comando, "rm ");
                strcat (comando, arctemp);
                system (comando);
             }   
          }
          clear();
          goto loop;
       }
       if ((c == KEY_F(1)) || (toupper(c) == 'H') || (c == '?')) {  /* Help screen   */
          hlp_scr();
          goto loop;         
       }
       if ( (toupper(c) == 'E')) {    /* richiama l'editor */
          if (sessionflag != 0) {   /* E' in sessione FTP */
             display23 (" Non puoi editare un file in una sessione FTP !!  (premi ENTER)");
             c = getch();
             goto loop;
          } 
          n = linecurs-1;
          sscanf(filename[n], "%s", stringa);
          l = strlen(stringa)-1;
          if (stringa[l] == '*') stringa[l] = 0; 
          if (fileauth[n][0] == '-')  {
             if (c == 'e') strcpy (comando, EDITOR1);
             else strcpy (comando, EDITOR2);
             strcat (comando, stringa);
             system (comando);
             clear();
          }
          goto loop;
       }   
       if ( toupper(c) == 'C') {    /* Copia file */
          if (sessionflag != 0) {   /* E' in sessione FTP */
             display23 (" Non puoi copiare un file in una sessione FTP !! (premi ENTER)");
             c = getch();
             goto loop;
          } 
          n = linecurs-1;
          if (fileauth[n][0] != '-')  {
             display23 (" Non puoi copiare una directory, un device o un link !!  (premi ENTER)");
             c = getch();
             goto loop;
          }   
          strcpy ( stringa, " Nome del nuovo file: ");
          display23 ( stringa );
          strcpy(stringa,"");
          move (23, 26);
          inputtext(stringa, 50, ' ');
          if (strlen(stringa) == 0) goto loop;
          if ((cp = rindex(filename[n], '*')) != NULL) *cp = ' ';
          strcpy (comando, "cp ");
          strcat (comando, filename[n]);
          strcat (comando, " ");
          strcat (comando, stringa);
          move (23, 0); refresh();
          if (system (comando) != 0) {
                display23(" Errore in CP        (premi ENTER) ");
                c = getch();
          }   
          else {      
             getdir();  
             videofill();
          }   
          clear();
          goto loop;
       }
       if ( (toupper(c) == 'D') || (c == KEY_DC) ) { 
          /* Cancellazione file o directory */
          n = linecurs-1;
          if (strncmp (filename[n], "tmp~", 4) == 0) {
             strcpy ( stringa, " Non puoi cancellare questo file temporaneo necessario a FL !!  (premi ENTER)");
             display23 ( stringa );
             c = getch();
             goto loop;
          }
          if ((cp = rindex(filename[n], '*')) != NULL) *cp = ' ';
          strcpy ( stringa, " Confermi la cancellazione di ");
          strcat ( stringa, filename[n]);
          strcat ( stringa, " ? (y/N)");
          display23 ( stringa );
          c = getch();
          if (toupper(c) == 'Y') {
             c = ' ';
             if (fileauth[n][0] == '-')  {
                if ((ftpflag == 1) && (sessionflag == 1)) 
                   strcpy (ftpcomando1, "delete ");
                else strcpy (comando, "rm ");
             }   
             if (fileauth[n][0] == 'd')  {
                if ((ftpflag == 1) && (sessionflag == 1)) 
                   strcpy (ftpcomando1, "rmdir ");
                else strcpy (comando, "rmdir ");
             }   
             if ((ftpflag == 1) && (sessionflag == 1)) {
                strcat (ftpcomando1, filename[n]);
                strcpy (ftpcomando, "nlist -alF ");
                strcat (ftpcomando, ftptemp);
                if (runftp() == 0) {
                   getdir(); linecurs = 1; nrec = 1;
                   videofill();
                   clear();
                }
                strcpy (ftpcomando1, "");
                goto loop;
             }
             else  strcat (comando, filename[n]);
             move (23, 0); refresh();
             if (system (comando) != 0) {
                display23(" Errore in RM / RMDIR        (premi ENTER) ");
                c = getch();
                clear();
             }   
             else {      
                getdir();
                videofill();
             }   
          }
       }
       if ( toupper(c) == 'P') {    /* Nuovo Path */
          display23 ( " Nuova directory: ");
          strcpy ( stringa, "" );
          move (23, 20); 
          inputtext(stringa, 58, ' ');
          if (strlen(stringa) == 0) goto loop;
          tagged = 0; for (i = 0; i < 2000 ; i++) filetag[i] = '\0';
          if ((ftpflag == 1) && (sessionflag == 1)) {
             strcpy (ftpworkdir, stringa);
             strcpy (ftpcomando, "nlist -alF ");
             strcat (ftpcomando, ftptemp);
             if (runftp() == 0) {
                getdir(); linecurs = 1; nrec = 1;
                videofill();
                clear();
             }
             goto loop;
          }   
          if (chdir(stringa) == 0) {             
             getdir (); linecurs = 1; nrec = 1;
             videofill();
          }
       }
       if ( toupper(c) == '/') {    /* Crea Directory */
          display23 ( " Crea directory: ");
          strcpy ( stringa, "" );
          move (23, 20); 
          inputtext(stringa, 58, ' ');
          if (strlen(stringa) == 0) goto loop;
          if ((ftpflag == 1) && (sessionflag == 1)) {
             strcpy (ftpcomando1, "mkdir ");
             strcat (ftpcomando1, stringa);
             strcpy (ftpcomando, "nlist -alF ");
             strcat (ftpcomando, ftptemp);
             runftp(); 
          }   
          else {
             strcpy (comando, "mkdir ");
             strcat (comando, stringa);
             move (23, 0); refresh();
             if (system (comando) != 0) {
                display23(" Errore in MKDIR        (premi ENTER) ");
                c = getch();
             }
          }   
          getdir(); linecurs = 1; nrec = 1;
          videofill();
          goto loop;
       }
       if ( (toupper(c) == 'R') || (toupper(c) == 'M') ) {    /* Rename */
          if ((sessionflag != 0) && (toupper(c) == 'M')) {   /* E' in FTP */
             display23 (" Non puoi spostare un file in una sessione FTP !! (premi ENTER)");
             c = getch();
             goto loop;
          } 
          n = linecurs-1;
          if (strncmp (filename[n], "tmp~", 4) == 0) {
             strcpy ( stringa, " Non puoi rinominare questo file temporaneo necessario a FL !!  (premi ENTER)");
             display23 ( stringa );
             c = getch();
             goto loop;
          }
          if (fileauth[n][0] != '-')  {
             display23 (" Non puoi rinominare una directory, un device o un link !!  (premi ENTER)");
             c = getch();
             goto loop;
          }   
          if (toupper(c) == 'R') strcpy ( stringa, " Rinomina il file in: ");
          else  strcpy ( stringa, " Sposta il file nella directory: ");
          display23 ( stringa );
          strcpy(stringa,"");
          move (23, 35); 
          inputtext(stringa, 42, ' ');
          if (strlen(stringa) == 0) goto loop;
          if ((cp = rindex(filename[n], '*')) != NULL) *cp = ' ';
          if ( (rindex(stringa, '/') != NULL) && (toupper(c) == 'R')) {
             display23 (" Usa 'M' (move) per spostare un file in un'altra directory.  (premi ENTER)");
             c = getch();
             goto loop;
          }
          if ( (rindex(stringa, '/') == NULL) && (toupper(c) == 'M')) {
             display23 (" Usa 'R' (rename) per cambiare nome a un file.  (premi ENTER)");
             c = getch();
             goto loop;
          }
          if ((ftpflag == 1) && (sessionflag == 1)) {
             strcpy (ftpcomando1, "rename ");
             strcat (ftpcomando1, filename[n]);
             strcat (ftpcomando1, " ");
             strcat (ftpcomando1, stringa);
             strcpy (ftpcomando, "nlist -alF ");
             strcat (ftpcomando, ftptemp);
             if (runftp() == 0) {
                getdir(); 
                videofill();
                clear();
             }
             strcpy (ftpcomando1, "");
             goto loop;   
          }
          strcpy (comando, "mv ");
          strcat (comando, filename[n]);
          strcat (comando, " ");
          strcat (comando, stringa);
          move (23, 0); refresh();
          if (system (comando) != 0) {
              display23(" Errore in MV        (premi ENTER) ");
              c = getch();
          }   
          else {      
             getdir();
             videofill();
          }   
          clear();
          goto loop;
       }
       if ( toupper(c) == 'I') {    /* Lancia il programma */
          if (sessionflag != 0) {   /* E' in sessione FTP */
             display23 (" Non puoi lanciare un programma in una sessione FTP !!  (premi ENTER)");
             c = getch();
             goto loop;
          } 
          n = linecurs-1;
          if ((cp = rindex(filename[n], '*')) != NULL) *cp = ' ';
          if ((fileauth[n][0] == '-') && (rindex(fileauth[n], 'x') != NULL)) {
             system (CLEAR);
             system(filename[n]);
             c = getch();
             clear();
          }   
       }
       if ( c == '!') {    /* Lancia comando in shell Unix */
          display23 ( " SHELL $ ");
          strcpy ( comando, "" );
          move (23, 10); 
          inputtext(comando, 68, ' ');
          if (strlen(comando) == 0) goto loop;
          strcpy (stringa, "echo ");
          strcat (stringa, comando);
          system (CLEAR);
          system (stringa);
          system (comando);
          c = getch();
          clear();
          goto loop;
       }
       if ( toupper(c) == 'S') {    /* Sort della directory */
          strcpy (sortflag, "");
          #ifdef AIX
             strcpy ( stringa, " Sort by: [-] descending (D)ate ");
          #else
             strcpy ( stringa, " Sort by: [-] descending (D)ate (S)ize (E)xtension ");
          #endif   
          display23 ( stringa ); 
          c = getch();
          if (c == '-') { strcpy (sortflag, "r"); c = getch(); }
          if (toupper(c) == 'D') strcat (sortflag, "t");
          #ifndef AIX
          if (toupper(c) == 'S') strcat (sortflag, "S");
          if (toupper(c) == 'E') strcat (sortflag, "X");
          #endif
          getdir();  linecurs = 1; nrec = 1;
          videofill();
       }
       if ( toupper(c) == 'T') {    /* Innesca sessione FTP */
          if (ftpflag != 0) {
             display23 (" Sessione FTP gia' attiva !!    (premi ENTER)");
             c = getch();
             goto loop;
          } 
          
          display23 ( " Inizio sessione FTP - Nome Host remoto: ");
          move (23, 44); 
          strcpy (stringa, ftphost);
          inputtext(stringa, 30, ' ');
          sscanf (stringa, "%s",ftphost);
          if (strlen(ftphost) == 0) goto loop;
          
          display23 ( " Nome Utente remoto: ");
          move (23, 22); 
          strcpy (stringa, ftpuser);
          inputtext(stringa, 16, ' ');
          sscanf (stringa, "%s",ftpuser);

          display23 ( " Password Utente remoto: ");
          move (23, 26); 
          strcpy (ftppassw, "");
          inputtext(ftppassw, 16, ' ');

          strcpy (ftpworkdir, "."); /* per ora, fa il list della home dir remota */
          strcpy (ftpcomando, "nlist -alF ");
          strcat (ftpcomando, ftptemp);
          move (23, 0); 
          if (runftp() == 0) {
             ftpflag = 1;
             sessionflag = 1;
             getdir(); linecurs = 1; nrec = 1;
             videofill();
             clear();
          }   
          else {
             display23 (" Errore in inizio sessione FTP !!  (premi ENTER)");
             c = getch();
             strcpy (comando, "rm ");
             strcat (comando, prmtemp);
             system (comando);
             clear();
             goto loop;
          }
       }
       if ( toupper(c) == 'W') {    /* Switch sessione FTP */
          if (ftpflag == 0) goto loop;
          else {
             if (sessionflag == 1) sessionflag = 0;
             else sessionflag = 1;
             getdir(); linecurs = 1; nrec = 1;
             nrecold = 0;   /* invalida nomi file presenti in memoria */
             videofill();
             clear();
          }
       }
       if ( toupper(c) == 'N') {    /* Fine sessione FTP */
          if (ftpflag == 0) goto loop;
          else {
             strcpy (comando, "rm ");
             strcat (comando, ftptemp);
             strcat (comando, " ");
             strcat (comando, msgtemp);
             strcat (comando, " ");
             strcat (comando, prmtemp);
             system (comando);
             ftpflag = 0;
             sessionflag = 0;
             getdir();  linecurs =1; nrec = 1;
             nrecold = 0;   /* invalida nomi file presenti in memoria */
             videofill();
             clear();
          }
       }
       if ( toupper(c) == 'V') {    /* Visualizzazione log FTP */
          if (ftpflag == 0) goto loop;
          else {
             strcpy (comando, PAGER1);
             strcat (comando, msgtemp);
             system (CLEAR);
             system (comando);
          }
          clear();
       }
       if ( toupper(c) == 'Y') {    /* Transfer PUT/GET con FTP */
          if (ftpflag == 0) {
             display23 (" Devi prima iniziare una sessione FTP.  (premi ENTER)");
             c = getch();
             goto loop;
          }   
          n = linecurs-1;
          if (fileauth[n][0] != '-')  {
             display23 (" Non puoi spedire una directory, un device o un link !!  (premi ENTER)");
             c = getch();
             goto loop;
          }   
          if ((cp = rindex(filename[n], '*')) != NULL) *cp = ' ';
          display23 (" Nome del file di arrivo: ");
          strcpy(stringa, filename[n]);
          move (23, 28);
          inputtext(stringa, 50, ' ');
          if (strlen(stringa) == 0) goto loop;
          if (sessionflag == 1) {   /* E' un ftp GET da remoto a locale  */ 
             strcpy (ftpcomando, "get ");
             strcat (ftpcomando, filename[n]);
             strcat (ftpcomando, " ");
             strcat (ftpcomando, stringa);
             if (runftp() == 0) {
                sessionflag = 0;
                getdir(); linecurs = 1; nrec = 1;
                videofill();
                clear();
             }
             goto loop;   
          }
          if (sessionflag == 0) {   /* E' un ftp PUT da locale a remoto  */ 
             strcpy (ftpcomando1, "put ");
             strcat (ftpcomando1, filename[n]);
             strcat (ftpcomando1, " ");
             strcat (ftpcomando1, stringa);
             strcpy (ftpcomando, "nlist -alF ");
             strcat (ftpcomando, ftptemp);
             if (runftp() == 0) {
                sessionflag = 1;
                getdir(); linecurs = 1; nrec = 1;
                videofill();
                clear();
             }
             strcpy (ftpcomando1, "");
             goto loop;   
          }
          
       }
       if ( toupper(c) == 'O') {    /* Cambia propietario (Owner) */
          if (sessionflag != 0) {   /* E' in sessione FTP */
             display23 (" Non puoi fare CHOWN file in una sessione FTP !! (premi ENTER)");
             c = getch();
             goto loop;
          } 
          n = linecurs-1;
          display23 ( " Nome nuovo propietario (Owner) : ");
          strcpy(stringa,"");
          move (23, 45); refresh();
          inputtext(stringa, 25, ' ');
          if (strlen(stringa) == 0) goto loop;
          strcpy (comando, "chown ");
          strcat (comando, stringa);
          strcat (comando, " ");
          strcat (comando, filename[n]);
          move (23, 0); refresh();
          if (system (comando) != 0) {
              display23(" Errore in CHOWN        (premi ENTER) ");
              c = getch();
          }   
          getdir();
          videofill();
          clear();
       }
       if ( toupper(c) == 'G') {    /* Cambia attributi (Chmod) */
          if (sessionflag != 0) {   /* E' in sessione FTP */
             display23 (" Non puoi fare CHMOD file in una sessione FTP !! (premi ENTER)");
             c = getch();
             goto loop;
          }   
          n = linecurs-1;
          display23 ( " Nuovi attributi file (Chmod) : ");
          strcpy(stringa,"");
          move (23, 45); refresh();
          inputtext(stringa, 10, ' ');
          if (strlen(stringa) == 0) goto loop;
          strcpy (comando, "chmod ");
          strcat (comando, stringa);
          strcat (comando, " ");
          strcat (comando, filename[n]);
          move (23, 0); refresh();
          if (system (comando) != 0) {
              display23(" Errore in CHMOD        (premi ENTER) ");
              c = getch();
          }   
          getdir();
          videofill();
          clear();
       }
       if ( c == '1') {    /* lista file in una sola colonna */
          if (fullscr == 0) {
             fullscr = 1;
             if (linecurs > 20 ) { nrec += 20; linecurs -= 20; }
             clear();
             videofill();
          }
       }
       if ( c == '2') {    /* lista file in due colonne      */
          if (fullscr == 1) {
             fullscr = 0;
             nrecold = 0;   /* invalida nomi file presenti in memoria */
             clear();
             videofill();
          }    
       }
       if (c == ' ') {
          n = nrec+linecurs-1;
          if (filetag[n] != '>') {
             filetag[n] = '>';
             tagged ++;
          }
          else {
             filetag[n] = ' ';
             tagged --;
          }
       }
               
       if ( (toupper(c) == 'Q') || (c == KEY_F(10)) || 
          (c == KEY_BREAK)  ) break;
       
loop: ;
    }   


fine: ;
    move(numrighe-1,0);
    refresh();
    endwin(); 
    /* Cancella file temporanei              */
    if (ftpflag != 0) {
       strcpy (comando, "rm ");
       strcat (comando, ftptemp);
       strcat (comando, " ");
       strcat (comando, msgtemp);
       strcat (comando, " ");
       strcat (comando, prmtemp);
       system (comando);
    }
    strcpy(comando, "rm ");
    strcat(comando, loctemp);
    if ( system(comando) != 0) {
       fprintf(stderr,"FL - Errore in comando: %s.\n", comando);
       exit (98);
    } 

    return 0;
}         

/* Help screen                         */

void hlp_scr()
{
     char c;

     werase(stdscr);
     attrset (A_NORMAL ); 
     
     mvaddstr(0, 28, " F I L E      L I S T E R");
     mvaddstr(1, 22, " Unix clone of Vernon Buerg's PC List");
     mvaddstr(2, 22, " v. 1.0 - (C) Piero Ronchi 10/09/1998");

     mvaddstr(4, 35, "MENU KEYS");
     mvaddstr(5, 1, "frecce-j/k    Posiziona sul nome file");
     mvaddstr(6, 1, "PgUp/PgDn-F/B Pagina prec./successiva");
     mvaddstr(7, 1, "ENTER         Visualizza file/archivio");
     mvaddstr(8, 1, "<escape>-F10  Fine programma");

     mvaddstr(5, 41, "HOME      Primo file dello schermo");
     mvaddstr(6, 41, "END       Ultimo file dello schermo");
     mvaddstr(7, 41, "ctrl-HOME Primo file");
     mvaddstr(8, 41, "ctrl-END  Ultimo file");

     mvaddstr(10, 33, "MENU COMMANDS");
     mvaddstr(11, 1, "A/G    Visualizza/cambia attributi");
     mvaddstr(12, 1, "C      Copia il file");
     mvaddstr(13, 1, "D      Cancella il file");
     mvaddstr(14, 1, "e/E    Edita col primo/secondo editor");
     mvaddstr(15, 1, "!      Esegue comando in shell Unix");
     mvaddstr(16, 1, "I      Invoca (esegue) il programma");
     mvaddstr(17, 1, "l/L    Vedi col primo/secondo browser");
     mvaddstr(18, 1, "M      Sposta il file");
     mvaddstr(19, 1, "P      Cambia il PATH corrente");
     mvaddstr(20, 1, "W      Swich con sessione FTP");

     mvaddstr(11, 42, "Q      Esce dal programma");
     mvaddstr(12, 42, "R      Rinomina il file");
     mvaddstr(13, 42, "S      Ordina i nomi files");
  /* mvaddstr(14, 42, "X      Esce nella directory corrente"); */
     mvaddstr(15, 42, "1-2    Numero di files per linea");
     mvaddstr(16, 42, "T/N    Inizio / Fine sessione FTP");
     mvaddstr(17, 42, "Y      Spedisce file locale con FTP");
     mvaddstr(18, 42, "V      Visualizza il log dell'FTP ");
     mvaddstr(19, 42, "O      Cambia il propietario[:gruppo]");
     mvaddstr(20, 42, "/      Crea nuova directory");

     mvaddstr(22, 1, "Gli archivi supportati sono: *.tar *.zip *.Z *.gz");

     mvaddstr(23, 28, "Premi ENTER per ritornare");

     refresh();     
     c = getch();  

     clear();
}
